from typing import Any

__all__ = [
    "_main",
    "freeze_support",
    "get_command_line",
    "get_executable",
    "get_preparation_data",
    "import_main_path",
    "set_executable",
]

def get_command_line(**kwds: Any) -> list[str]: ...
def get_executable() -> str: ...
def get_preparation_data(name: str) -> dict[str, Any]: ...
def import_main_path(main_path: str) -> None: ...
def set_executable(exe: str) -> None: ...
def freeze_support() -> None: ...
def _main(fd: int) -> None: ...
