from typing import Any

__all__ = ("SessionManager",)

class SessionManager:
    def __init__(self) -> None: ...
    def get_engine(self, dburi: str, **kwargs: Any) -> Any: ...
    def create_session(
        self, dburi: str, short_lived_sessions: bool = False, **kwargs: Any
    ) -> Any: ...
    def prepare_models(self, engine: Any) -> None: ...
    def session_factory(self, dburi: str, **kwargs: Any) -> Any: ...
