from typing import Any

__all__ = ("Pidbox", "gPidbox")

class Pidbox:
    consumer: Any
    connection: Any
    node: Any

    def __init__(self, c: Any) -> None: ...
    def on_stop(self) -> None: ...
    def start(self, c: Any) -> None: ...
    def stop(self, c: Any) -> None: ...
    def reset(self) -> None: ...
    def shutdown(self, c: Any) -> None: ...
    def on_message(self, body: Any, message: Any) -> None: ...

class gPidbox(Pidbox):
    def __init__(self, c: Any) -> None: ...
    def loop(self, c: Any) -> None: ...
