from typing import Any

from kombu.message import Message

class BaseAsyncMessage(Message): ...
class AsyncRawMessage(BaseAsyncMessage): ...

class AsyncMessage(BaseAsyncMessage):
    def encode(self, value: str) -> str: ...
    def __getitem__(self, item: str) -> Any: ...
