from typing import Any

__all__ = ("Worker",)

from celery.worker import WorkController
from typing_extensions import override

class Worker(WorkController):
    def emit_banner(self) -> None: ...
    def extra_info(self) -> str: ...
    def install_platform_tweaks(self, worker: WorkController) -> None: ...
    def macOS_proxy_detection_workaround(self) -> None: ...
    @override
    def on_after_init(
        self,
        purge: bool = False,
        no_color: bool | None = None,
        redirect_stdouts: bool | None = None,
        redirect_stdouts_level: str | None = None,
        **kwargs: Any,
    ) -> None: ...
    @override
    def on_before_init(self, quiet: bool = False, **kwargs: Any) -> None: ...
    @override
    def on_consumer_ready(self, consumer: Any) -> None: ...
    @override
    def on_init_blueprint(self) -> None: ...
    @override
    def on_start(self) -> None: ...
    def purge_messages(self) -> int: ...
    def set_process_status(self, info: str) -> None: ...
    def setup_logging(self, colorize: bool | None = None) -> None: ...
    def startup_info(self, artlines: bool = True) -> str: ...
    def tasklist(
        self, include_builtins: bool = True, sep: str = "\n", int_: str = "celery."
    ) -> str: ...
