from typing import Any

__all__ = ("CursesMonitor", "evtop")

class CursesMonitor:
    keymap: dict[Any, Any]
    keyalias: dict[Any, Any]
    win: Any
    selected_position: int
    screen_delay: float
    background: int
    foreground: int
    greet: str
    help: str

    def __init__(self, state: Any, app: Any, keymap: Any = None) -> None: ...
    def alert(self, callback: Any, title: str | None = None) -> None: ...
    def alert_remote_control_reply(self, reply: Any) -> None: ...
    @property
    def display_height(self) -> int: ...
    def display_task_row(self, lineno: int, task: Any) -> None: ...
    @property
    def display_width(self) -> int: ...
    def draw(self) -> None: ...
    def find_position(self) -> int: ...
    def format_row(
        self, uuid: str, task: Any, worker: Any, timestamp: Any, state: Any
    ) -> str: ...
    def handle_keypress(self) -> bool | None: ...
    @property
    def help_title(self) -> str: ...
    @property
    def info_str(self) -> str: ...
    def init_screen(self) -> None: ...
    @property
    def limit(self) -> int: ...
    def move_selection(self, direction: int = 1) -> None: ...
    def move_selection_down(self) -> None: ...
    def move_selection_up(self) -> None: ...
    def nap(self) -> None: ...
    @property
    def online_str(self) -> str: ...
    def readline(self, x: int, y: int) -> str: ...
    def resetscreen(self) -> None: ...
    def revoke_selection(self) -> None: ...
    def safe_add_str(self, y: int, x: int, string: str, *args: Any) -> None: ...
    @property
    def screen_height(self) -> int: ...
    @property
    def screen_width(self) -> int: ...
    @property
    def selected_str(self) -> str: ...
    @property
    def selected_task(self) -> Any: ...
    def selection_info(self) -> None: ...
    def selection_rate_limit(self) -> None: ...
    def selection_result(self) -> None: ...
    def selection_traceback(self) -> None: ...
    @property
    def tasks(self) -> list[Any]: ...
    @property
    def workers(self) -> list[Any]: ...

def evtop(app: Any | None = None) -> None: ...
