from queue import Empty as Empty
from queue import Full as Full
from typing import Any, Generic, TypeVar

__all__ = ["JoinableQueue", "Queue", "SimpleQueue"]

_T = TypeVar("_T")

class Queue(Generic[_T]):
    def __init__(self, maxsize: int = ..., *args: Any, **kwargs: Any) -> None: ...
    def get(self, block: bool = ..., timeout: float | None = ...) -> _T: ...
    def put(self, obj: _T, block: bool = ..., timeout: float | None = ...) -> None: ...
    def get_nowait(self) -> _T: ...
    def put_nowait(self, obj: _T) -> None: ...
    def empty(self) -> bool: ...
    def full(self) -> bool: ...
    def qsize(self) -> int: ...
    def close(self) -> None: ...
    def join_thread(self) -> None: ...
    def cancel_join_thread(self) -> None: ...

class JoinableQueue(Queue[_T]):
    def __init__(self, maxsize: int = ..., *args: Any, **kwargs: Any) -> None: ...
    def task_done(self) -> None: ...
    def join(self) -> None: ...

class SimpleQueue(Generic[_T]):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def get(self) -> _T: ...
    def put(self, obj: _T) -> None: ...
    def empty(self) -> bool: ...
    def close(self) -> None: ...
    def get_payload(self) -> Any: ...
    def send_payload(self, value: Any) -> None: ...
