__all__ = ["BufferWrapper"]

class BufferWrapper:
    def __init__(self, size: int) -> None: ...
    def create_memoryview(self) -> memoryview: ...
    def get_address(self) -> int: ...
    def get_size(self) -> int: ...
