from typing import Any

__all__ = ()

def add_backend_cleanup_task(app: Any) -> Any: ...
def add_accumulate_task(app: Any) -> Any: ...
def add_chain_task(app: Any) -> Any: ...
def add_chord_task(app: Any) -> Any: ...
def add_chunk_task(app: Any) -> Any: ...
def add_group_task(app: Any) -> Any: ...
def add_map_task(app: Any) -> Any: ...
def add_starmap_task(app: Any) -> Any: ...
def add_unlock_chord_task(app: Any) -> Any: ...
def connect_on_app_finalize(callback: Any) -> Any: ...
