from collections.abc import Iterable, Iterator
from typing import NamedTuple

class version_info_t(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: str
    serial: str

def version_string_as_tuple(s: str) -> version_info_t: ...
def fmatch_iter(
    needle: str, haystack: Iterable[str], min_ratio: float = ...
) -> Iterator[tuple[float, str]]: ...
def fmatch_best(
    needle: str, haystack: Iterable[str], min_ratio: float = ...
) -> str | None: ...
def escape_regex(p: str, white: str = ...) -> str: ...
