from typing import Any

def annotation_is_class(annotation: Any) -> bool: ...
def annotation_issubclass(annotation: Any, cls: type) -> bool: ...
def get_optional_arg(annotation: Any) -> Any: ...
def is_none_type(value: Any) -> bool: ...
