from typing import Any

__all__ = ("MapAnnotation", "prepare", "resolve_all")

class MapAnnotation:
    def __init__(self, d: dict[str, Any]) -> None: ...
    def annotate(self, task: Any) -> dict[str, Any] | None: ...
    def annotate_any(self) -> dict[str, Any] | None: ...

def prepare(annotations: Any) -> Any: ...
def resolve_all(anno: Any, task: Any) -> dict[str, Any]: ...
