TRANSPORT_ALIASES: dict[str, str]

def get_transport_cls(transport: str | None = ...) -> str | None: ...
def resolve_transport(transport: str | None = ...) -> str | None: ...
def supports_librabbitmq() -> bool | None: ...
