from typing import Any

__all__ = ("TaskRegistry",)

from celery.app.task import Task
from celery.exceptions import NotRegistered

class TaskRegistry(dict[str, Task[Any, Any]]):
    NotRegistered: type[NotRegistered]

    def __missing__(self, key: str) -> Task[Any, Any]: ...
    def filter_types(self, type: str) -> dict[str, Task[Any, Any]]: ...
    def periodic(self) -> dict[str, Task[Any, Any]]: ...
    def register(self, task: Task[Any, Any]) -> Task[Any, Any]: ...
    def regular(self) -> dict[str, Task[Any, Any]]: ...
    def unregister(self, name: str) -> None: ...
