from kombu.asynchronous.hub import Hub

def callback_for(h: Hub, fd: int, flag: int, *default: str) -> str: ...
def repr_flag(flag: int) -> str: ...
def repr_active(h: Hub) -> str: ...
def repr_events(h: Hub, events: list[tuple[int, int]]) -> str: ...
def repr_readers(h: Hub) -> list[str]: ...
def repr_writers(h: Hub) -> list[str]: ...
