from typing import Any

__all__ = ("colored",)

class colored:
    def __init__(self, *s: object, **kwargs: Any) -> None: ...
    def __add__(self, other: object) -> str: ...
    def embed(self) -> str: ...
    def no_color(self) -> str: ...
    def node(self, s: tuple[object, ...], op: str) -> colored: ...
    def reset(self, *s: object) -> colored: ...
    def black(self, *s: object) -> colored: ...
    def blink(self, *s: object) -> colored: ...
    def blue(self, *s: object) -> colored: ...
    def bold(self, *s: object) -> colored: ...
    def bright(self, *s: object) -> colored: ...
    def cyan(self, *s: object) -> colored: ...
    def green(self, *s: object) -> colored: ...
    def iblue(self, *s: object) -> colored: ...
    def icyan(self, *s: object) -> colored: ...
    def igreen(self, *s: object) -> colored: ...
    def imagenta(self, *s: object) -> colored: ...
    def ired(self, *s: object) -> colored: ...
    def iwhite(self, *s: object) -> colored: ...
    def iyellow(self, *s: object) -> colored: ...
    def magenta(self, *s: object) -> colored: ...
    def red(self, *s: object) -> colored: ...
    def reverse(self, *s: object) -> colored: ...
    def underline(self, *s: object) -> colored: ...
    def white(self, *s: object) -> colored: ...
    def yellow(self, *s: object) -> colored: ...
