from datetime import datetime
from typing import Any

from celery.app.base import Celery

__all__ = ("DjangoFixup", "fixup")

class DjangoFixup:
    app: Celery
    def __init__(self, app: Celery) -> None: ...
    def install(self) -> None: ...
    def autodiscover_tasks(self) -> None: ...
    def now(self, utc: bool = False) -> datetime: ...
    def on_import_modules(self, **kwargs: Any) -> None: ...
    def on_worker_init(self, **kwargs: Any) -> None: ...
    @property
    def worker_fixup(self) -> Any: ...

def fixup(app: Celery, env: str = "DJANGO_SETTINGS_MODULE") -> DjangoFixup | None: ...
