import os
from typing import Any

__all__ = ("df", "load_average")

class df:
    def __init__(self, path: str | bytes | os.PathLike[Any]) -> None: ...
    @property
    def available(self) -> float: ...
    @property
    def capacity(self) -> int: ...
    @property
    def stat(self) -> os.statvfs_result: ...
    @property
    def total_blocks(self) -> float: ...

def load_average() -> tuple[float, float, float] | None: ...
